#!/usr/bin/perl -w
use strict;

##############################################################
# This script is to compute the local accuracy of models and 
# predict the normalized B-factor of a query protein.
#
# Usage:
#
# ./runResQ_IT dir_of_I-TASSER_decoy
#
# example:
# ./runResQ_IT.pl /home/zhanglabs/I-TASSER/data/example
##############################################################

if(@ARGV<1)
{
    print "./runResQ_IT.pl dir_of_I-TASSER_decoy\n\n";
    exit;
}

my $cluster_dir=$ARGV[0]; #It should contain: seq.dat.ss, init.dat, rst.dat, str.txt, rep*, combo*.pdb, model*.pdb

#my $user   = "yangji";


my $user   = `whoami`;
chomp($user);


my $nc_max = 5; #cluculate RSQ for 5 models
my $dcut   = 5; #cutoff for decoy numbers in a cluster


my  $ssf="seq.dat.ss";
if(!-s "$cluster_dir/$ssf")
{
    $ssf="seq.ss";
}

`cp $cluster_dir/cluster/tra.in $cluster_dir/` if(-s "$cluster_dir/cluster/tra.in");
`cp $cluster_dir/cluster/str.txt $cluster_dir/` if(-s "$cluster_dir/cluster/str.txt");
`cp $cluster_dir/cluster/rst.dat $cluster_dir/` if(-s "$cluster_dir/cluster/rst.dat");
`cp $cluster_dir/cluster/combo*.pdb $cluster_dir/` if(-s "$cluster_dir/cluster/combo1.pdb");

if(!-s "$cluster_dir/tra.in")
{
    chdir $cluster_dir;
    my @rst=`ls rep*.bz2`;
    chomp(@rst);
    my $n=@rst;
    
    open(TTT, ">tra.in");
    print TTT "$n\n";
    foreach my$r(@rst)
    {
	print TTT "$r\n";
    }
    close(TTT);
}


for(my $i=1; $i<=$nc_max; $i++)
{
    if(!-s "$cluster_dir/model$i.pdb")
    {
	$nc_max=$i-1;
	last;
    }
}

if($nc_max<1)
{
    print "No model*.pdb exists at $cluster_dir, exit\n";
    exit;
}



##### report node -------->
`hostname`=~/(\S+)/;
my $node=$1;
printf "hostname: $node\n";
my $time=`date`;
printf "starting time: $time";
my $pwd=`pwd`;
printf "pwd: $pwd";
#^^^^^^^^^^^^^^^^^^^^^^^^^^
my $home="/nfs/amino-home";
if(!-d $home)
{
    $home="/home";
}

#my $bindir="/home/yzhang/TASSER_server_casp9/bin"; #where the rmsdf program is located
my $bindir="$home/zhanglabs/I-TASSER/bin/local"; #where the rmsdf, stride, and svm programs are located
my $libdir="$home/yangji/project/RC/Bfactor"; #where the bfactor library is

my %ts=(
     'GLY'=>'G',
     'ALA'=>'A',
     'VAL'=>'V',
     'LEU'=>'L',
     'ILE'=>'I',
     'SER'=>'S',
     'THR'=>'T',
     'CYS'=>'C',
     'MET'=>'M',
     'PRO'=>'P',
     'ASP'=>'D',
     'ASN'=>'N',
     'GLU'=>'E',
     'GLN'=>'Q',
     'LYS'=>'K',
     'ARG'=>'R',
     'HIS'=>'H',
     'PHE'=>'F',
     'TYR'=>'Y',
     'TRP'=>'W',

     'ASX'=>'A',
     'GLX'=>'G',
     'UNK'=>'X',

     'G'=>'GLY',
     'A'=>'ALA',
     'V'=>'VAL',
     'L'=>'LEU',
     'I'=>'ILE',
     'S'=>'SER',
     'T'=>'THR',
     'C'=>'CYS',
     'M'=>'MET',
     'P'=>'PRO',
     'D'=>'ASP',
     'N'=>'ASN',
     'E'=>'GLU',
     'Q'=>'GLN',
     'K'=>'LYS',
     'R'=>'ARG',
     'H'=>'HIS',
     'F'=>'PHE',
     'Y'=>'TYR',
     'W'=>'TRP',

     'a'=>'CYS',
     'b'=>'CYS',
     'c'=>'CYS',
     'd'=>'CYS',
     'e'=>'CYS',
     'f'=>'CYS',
     'g'=>'CYS',
     'h'=>'CYS',
     'i'=>'CYS',
     'j'=>'CYS',
     'k'=>'CYS',
     'l'=>'CYS',
     'm'=>'CYS',
     'n'=>'CYS',
     'o'=>'CYS',
     'p'=>'CYS',
     'q'=>'CYS',
     'r'=>'CYS',
     's'=>'CYS',
     't'=>'CYS',
     'u'=>'CYS',
     'v'=>'CYS',
     'w'=>'CYS',
     'x'=>'CYS',
     'y'=>'CYS',
     'z'=>'CYS',

     'B'=>'ASX',
     'Z'=>'GLX',
     'X'=>'CYS',
     );

my %SA=(
        "A"=>110.2,
        "R"=>229.0,
        "N"=>146.4,
        "D"=>144.1,
        "C"=>140.4,
        "Q"=>178.6,
        "E"=>174.7,
        "G"=>78.7,
        "H"=>181.9,
        "I"=>185.0,
        "L"=>183.1,
        "K"=>205.7,
        "M"=>200.1,
        "F"=>200.7,
        "P"=>141.9,
        "S"=>117.2,
        "T"=>138.7,
        "W"=>240.5,
        "Y"=>213.7,
        "V"=>153.7,
        );




my @files=("str.txt", "rst.dat", "pssm.txt", "init.dat", "tra.in", "combo1.pdb", "model1.pdb", "exp.dat");
push(@files, $ssf);

foreach my $f(@files)
{
    if(!-s "$cluster_dir/$f")
    {
	print "$cluster_dir/$f is missed, exit\n";
	exit;
    }
}

my @files1=("rmsdf", "stride", "svm_classify", "svm_model_bfp_noTM", "svm_model_good_noTM", "svm_model_bad_noTM", "align");
foreach my $f(@files1)
{
    if(!-s "$bindir/$f")
    {
	print "$bindir/$f is missed, exit\n";
	exit;
    }
}

my $random=int(1000000000*rand);
my $tag= "RC_$random";
my $workdir="/tmp/$user/$tag";
`mkdir -p $workdir`;
`rm -fr $workdir/*`;
chdir $workdir;
print "$workdir\n";

foreach my $f(@files)
{    
    `cp $cluster_dir/$f .`;
}
foreach my $f(@files1)
{
    `cp $bindir/$f .`;
}


my %seq=();
my %sec=();
my $len=0;
my @rst=`cat $ssf`;

foreach my $line(@rst)
{
    if($line =~ /\s*(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
    {
	$seq{$1}   =$2;
	$sec{$1, 0}=$3;
	$sec{$1, 1}=$4;
	$sec{$1, 2}=$5;
	$sec{$1, 3}=$6;
	$len++;

	if($len != $1)
	{
	    print "wrong format of $ssf\n";
	    exit;
	}
    }
}
my $Lenth=$len;

print "predict B-factor...\n";
my ($rsq_ref, $bfp_ref)=&BFP($Lenth, "init.dat", "pssm.txt", $ssf, "exp.dat");
my %bfp=%$bfp_ref;
my %rsa=%$rsq_ref;

print "compute coverage...\n";
my ($covt_ref, $covg_ref)=&read_threading_cov("init.dat", $Lenth, 10); ##compute both top 10 templates and global coverages

my %covt=%$covt_ref;
my %covg=%$covg_ref;

##get global C-score
my $cscore_g=-5;
my @global=`$bindir/get_cscore1.pl $cluster_dir`;
my @rmsd=();
chomp(@global);
foreach my $r(@global)
{
    if($r=~/Model\d+:\s+(\S+)\s+(\S+)\+.+\s+(\S+)\+.+/)
    {
	$cscore_g=$1;
        my $prm=$3;
        push(@rmsd, $prm);
        #print "$prm\n";
    }
}

print "compute RSQ...\n";
my %rsq=&RSQ($cscore_g, $random, "$workdir/init.dat", \%covg, \%covt, \%rsa);


#rescale to be consitent with the global estimation of RMSD

my %ave=();
for(my $j=1; $j<=$nc_max; $j++)
{
    $ave{$j}=0;
}
for(my $i=1; $i<=$Lenth; $i++)
{
    for(my $j=1; $j<=$nc_max; $j++)
    {
         $ave{$j}+=$rsq{$j, $i, 1};
    }
}
for(my $j=1; $j<=$nc_max; $j++)
{ 
    $ave{$j} /= $Lenth;  
}
my %ratio=();
my %added=();
$added{1}=0;
for(my $j=1; $j<=$nc_max; $j++)
{         
    $ratio{$j}=$ave{$j}/$rmsd[$j-1];
    $ratio{$j}=1 if($ratio{$j}<1);

    if($j==1)
    {
	if($ave{$j}>8) #adjust model1 for big distance
	{
	    $ratio{$j}=1.5 if($ratio{$j}<1.5);
	}
    }
    else
    {
	if($ave{1}/$ratio{1} > $ave{$j}/$ratio{$j} ) #model1 should be the best
	{
	    $added{$j}=$ave{1}/$ratio{1} - $ave{$j}/$ratio{$j}+0.5;
	    if($ratio{$j}==1)
	    {
		my $a=($rmsd[$j-1]-$ave{$j})/2;
		$added{$j} = $a if($added{$j}< $a);
	    }	    
	}
	else
	{
	    $added{$j}=0;
	}
    }

    printf "$j\t%.2f\t%.2f\t%.2f\t%.2f\n",$ave{$j}, $rmsd[$j-1], $ratio{$j}, $added{$j};
}



open(OUT, ">rsq_bfp_new.dat");
print OUT "#RES\t0\tSS\tSA\tCOV\tBFP\tRBF\t";
for(my $i=1; $i<=$nc_max; $i++)
{
    #print OUT "RSQ_$i\tSTD_$i\t";
    print OUT "RSQ_$i\t";
}
print OUT "\n";
for(my $i=1; $i<=$Lenth; $i++)
{    
    printf OUT "$i\t0\t%d\t%d\t%.2f\t%.2f\t%.2f\t", $bfp{$i, 1}, $bfp{$i, 2}, $covg{$i}, $bfp{$i, 3}, $bfp{$i, 4};

    for(my $j=1; $j<=$nc_max; $j++)
    {
	#printf OUT "%.2f\t%.2f\t", $rsq{$j, $i, 1}, $rsq{$j, $i, 2};
	printf OUT "%.2f\t", $rsq{$j, $i, 1}/$ratio{$j}+$added{$j};
    }

    print OUT "\n";
}
close(OUT);

`cp rsq_bfp_new.dat $cluster_dir/`;


sleep(1);
print "done\n";
`rm -fr $workdir`;
exit;


sub RSQ
{
###### copy trajectories -------->
    my ($cscore_g, $target, $init, $covg_ref, $covt_ref, $rsa_ref) = @_;

    my %covg=%$covg_ref;
    my %covt=%$covt_ref;
    my %rsa=%$rsa_ref;

    my %rsq=();
    open(RST, "rst.dat");
    my %density=();
    my %Rcin=();
    my $nstr=1;
    my $nc=0;
    my $cut=0.8;
    while(my $line=<RST>)
    {	
	$nstr=$1 if($line=~/Number of structure in use=\s*(\d+)/);
	$nc=$1 if($line=~/Number of clusters:\s*(\d+)/);
	$nc=$nc_max if($nc>$nc_max);
    
	if($line=~/C-----/)
	{
	    for(my $i=1;$i<=$nc;$i++)
	    {
		$line=<RST>;
		$line=~/(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
		my $Nin=$2;
		$Rcin{$i}=$4;
		$Rcin{$i}=$cut if($Rcin{$i}<$cut);
		$density{$i}=$Nin/$nstr;
	    }
        }
    }

    close(RST);



    open(A, "tra.in");
    <A>=~/(\d+)/;
    my $ntr=$1;
    for(my $i=1;$i<=$ntr;$i++){
	my $line=<A>;
	if($line=~/(\S+)/){
	    my $tra=$1;
	    #printf "cp $cluster_dir/$tra .\n";
	    if(-s "$cluster_dir/$tra")
	    {
		`cp $cluster_dir/$tra .`;
	    }
	    elsif(-s "$cluster_dir/$tra.bz2")
	    {
		`cp $cluster_dir/$tra.bz2 .`;
		`bzip2 -d $tra.bz2`;
		`rm -f $tra.bz2`;
	    }	
	    else
	    {
		print "$cluster_dir/$tra is missed\n";
		exit;
	    }
	}
    }
    close(A);

    
    open(IN, "str.txt");
    while(my $line =<IN>)
    {    
	if($line =~ /\#Cluster\s+(\d+)/)
	{
	    my $cluster=$1;		
	    last if($cluster>$nc_max);
	    
	    print "$line";
	    
	    my $refmod="combo$cluster.pdb";  #change here to closc or combo
	    my $ref=$refmod;
	    $ref =~ s/\.pdb//;
	    
	    if(-s "$cluster_dir/$refmod")
	    {
		`cp $cluster_dir/$refmod .`;
	    }
	    elsif(-s "$cluster_dir/cluster/$refmod")
	    {
		`cp $cluster_dir/cluster/$refmod .`;
	    }
	    else
	    {
		print "$cluster_dir/$refmod is missed\n";
		if($cluster==1)
		{
		    exit;
		}
		next;
	    }
	    
	    my $nr=0;
	    my $refmod1="tttt.pdb";
	    open(REF, "$refmod");
	    open(OUT, ">$refmod1");
	    while(my $line = <REF>)
	    {
		if($line =~ /^ATOM/)
		{    
		    next if(substr($line, 12, 4) ne " CA ");
		    $nr++;	
		    print OUT $line;
		}
	    }
	    close(REF);
	    close(OUT);
	    
	    if($Lenth != $nr)
	    {
		print "wrong length in $refmod $Lenth != $nr";
		exit;
	    }
	    
	    
	    my %from=();
	    my %std=();
	    my %s1=();
	    my %s2=();
	    for(my $i=1; $i<=$Lenth; $i++)
	    {
		$s1{$i}=0;
		$s2{$i}=0;
	    }
	    <IN>;
	    <IN>;
	    $line =<IN>;
	    my $nstr=0;
	    if($line =~ /^\s*Nstr=\s*(\d+)/)
	    {
		$nstr=$1;    
		for(my $i=1; $i<=$nstr; $i++)
		{
		    $line=<IN>;
		    chomp($line);
		    if($line =~ /\s+(\d+)\s+(\S+)\s*$/)
                    {			
                        my $tra=$2;
                        my $n=$1;

			push(@{$from{$tra}}, $n);
		    }
		    else
		    {
			print "wrong format in str.txt\n";
			exit;
		    }
		}
	    }
	    
	    ##analyze the structure deviation for each residue using combo or closc structure as reference
	    
	    ## step1, read the decoy structures and superimpose them onto reference structure
	    print "read and superimpose decoy structure...\n";
	    open(OUT, ">cluster$cluster.dat");
	    my $nstr1=0;
	    my @keys=keys %from;	
	    foreach my $tra(@keys)
	    {
		print "$tra\n";
		my %used=();
		my @str=@{$from{$tra}};
		foreach my $i(@str)
		{
		    $i=sprintf("%d", $i);
		    $used{$i}=1;
		    #print "$i\n";
		}
		
		my $i_str=0;
		open(TRA, "$tra");
		while($line=<TRA>)
		{	
		    if($line=~ /^\s*(\d+)\s+\S+/)
		    {
			my $Lch=$1;		    
			$i_str++;
			my $flag=0;
			if(exists $used{$i_str})
			{
			    $flag=1;
			    $nstr1++;
			    print OUT "$Lch $nstr1\n";
			}
			if($flag==1)
			{
			    my %xyz=();
			    my $file="$i_str.pdb";
			    open(TMP, ">$file");
			    for(my $j=1; $j<=$Lch; $j++)
			    {
				$line=<TRA>;			
				if($line =~ /(\S+)\s+(\S+)\s+(\S+)/)
				{
				    my $x=$1; 
				    my $y=$2;
				    my $z=$3;
				    $xyz{$j, 1}=$x; $xyz{$j, 2}=$y; $xyz{$j, 3}=$z;
				    
				    printf TMP "ATOM  %5d  CA  %3s  %4d    %8.3f%8.3f%8.3f\n", $j, $ts{$seq{$j}}, $j, $x, $y, $z;
				}
				else
				{
				    print "wrong format x y z in $tra\n";
				    exit;
				}
			    }
			    close(TMP);
			    
			    
			    ##superimposed structure
			    my @tra=();
			    my @rot=();
			    my $rst=`./rmsdf $i_str.pdb $refmod1`;
			    #print "$rst";exit;
			    if($rst =~/\s+1\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
			    {
				$tra[0]=$1;
				$rot[0][0]=$2;
				$rot[0][1]=$3;
				$rot[0][2]=$4;
				#printf "%.5f,%.5f,%.5f,%.5f\n", $1,$2,$3,$4;
			    }
			    if($rst =~/\s+2\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
			    {
				$tra[1]=$1;
				$rot[1][0]=$2;
				$rot[1][1]=$3;
				$rot[1][2]=$4;
				#printf "%.5f,%.5f,%.5f,%.5f\n", $1,$2,$3,$4;   
			    }
			    if($rst =~/\s+3\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
			    {
				$tra[2]=$1;
				$rot[2][0]=$2;
				$rot[2][1]=$3;
				$rot[2][2]=$4;
				#printf "%.5f,%.5f,%.5f,%.5f\n", $1,$2,$3,$4;
			    }
			    
			    for(my $j=1; $j<=$Lch; $j++)
			    {
				my $x=$xyz{$j, 1}; 
				my $y=$xyz{$j, 2}; 
				my $z=$xyz{$j, 3}; 
				

				my $rx = $rot[0][0]*$x + $rot[0][1]*$y + $rot[0][2]*$z + $tra[0];
				my $ry = $rot[1][0]*$x + $rot[1][1]*$y + $rot[1][2]*$z + $tra[1];
				my $rz = $rot[2][0]*$x + $rot[2][1]*$y + $rot[2][2]*$z + $tra[2];
				
				printf OUT "%10.3f %10.3f %10.3f\n", $rx, $ry, $rz;		    			    
			    }			
			    `rm -f $file`;
			}
			else
			{
			    for(my $j=1; $j<=$Lch; $j++)
			    {
				$line=<TRA>;
			    }
			}
		    }		
		    else
		    {
			print "wrong format head in $tra\n$line\n";
			last;
		    }
		}
		close(TRA);
		
		#last;
	    }	
	    close(OUT);
	    
	    
	    ##compute std
	
	    if($nstr1!=$nstr)
	    {
		print "Warning: nstr1($nstr1) != nstr($nstr)\n";
	    }	
	    print "Calculate cluster std...\n";
	    %std=&compute_std($refmod, "cluster$cluster.dat", $Lenth);
	    
	    print "assign SS with stride for model$cluster.pdb..\n";
	    if(!-s "$cluster_dir/model$cluster.pdb")
	    {
		print "$cluster_dir/model$cluster.pdb is missed\n";
		exit;		
	    }
	    `cp $cluster_dir/model$cluster.pdb $workdir/`;
	    my %ss=&make_ss("$workdir/model$cluster.pdb", $Lenth);

	    my ($std1_ref) = &read_threading_dist($init, $Lenth, 10, "$workdir/model$cluster.pdb", $target); #dist to top ten templates
	    my %std1  = %$std1_ref;
	    
	    print "predict model local errors with svm...\n";
	    open(FFF, ">feature.dat");	   
	    
	    for(my $i=1; $i<=$Lenth; $i++)
	    {



		my $score =0;
		my $score1=0;
		
		my $asa=0;

		if(exists $ss{$i, 1}) #in case stride fails
                {
		    $asa=$ss{$i, 2};
                    if($ss{$i, 1} eq $sec{$i, 0})
                    {
                        $score=1;			
                    }
                    else
                    {
                        if($sec{$i, 0} eq "C")
                        {
                            $score=0.5;
                        }
                    }
                }
		my $diff= abs($rsa{$i} - $asa);
		$score1=1/(1+$diff);		

		my $nf=1;
		print FFF "0 ";
		    
		printf FFF "%d:%.3f ", $nf, $std{$i, 3}; $nf++; ##mean(d)    
		printf FFF "%d:%.3f ", $nf, $std{$i, 2}; $nf++; ##standard deviation			
		printf FFF "%d:%.3f ", $nf, $covg{$i};  $nf++;  ##global coverage
		printf FFF "%d:%.3f ", $nf, $covt{$i};  $nf++;  ##top coverage

		printf FFF "%d:%.3f ", $nf, $std1{$i}; $nf++;  ##dist between model and top LOMETS templates

		printf FFF "%d:%.3f ", $nf, $sec{$i, 1};  $nf++;   ##secondary structure profile 
		printf FFF "%d:%.3f ", $nf, $sec{$i, 2};  $nf++;   ##secondary structure profile 
		printf FFF "%d:%.3f ", $nf, $sec{$i, 3};  $nf++;   ##secondary structure profile
		
		printf FFF "%d:%.3f ", $nf, $score; $nf++;
		printf FFF "%d:%.3f ", $nf, $score1; $nf++;

		printf FFF"%d:%.3f ", $nf, $density{$cluster};$nf++;
		printf FFF "\n";		
	    }
	    close(FFF);

	    `cp feature.dat $cluster_dir/feature_$cluster.dat`;	    
	    
	    my $modelname="svm_model_good_noTM"; #svm_small_cluster
	    if($cscore_g<-1.5) 
	    {    
		$modelname="svm_model_bad_noTM";
	    }
	    
	    `./svm_classify feature.dat $modelname feature.pred >ttt`;
	    #`cp feature.dat feature_$cluster.dat`;
	    
	    my @pred=`cat feature.pred`;
	    chomp(@pred);
	    open(STD, ">std_results.dat");
	    for(my $i=1; $i<=$Lenth; $i++)
	    {
		my $j=$i-1;
		#$pred[$j]=0.1 if($pred[$j]<0.1);
		$pred[$j]=$std{$i, 3} if($pred[$j]<0.5);
		$pred[$j]=0.1 if($pred[$j]<0.1);
		
		$rsq{$cluster, $i, 1}=$pred[$j];
		$rsq{$cluster, $i, 2}=$std{$i, 2};

		printf STD "%d\t%.3f\t%.3f\n",  $i, $pred[$j], $std{$i, 2};
	    }
	    close(STD);
	    
	    #`cp std_results.dat $cluster_dir/RC_$cluster.dat`;
	}
    }
    close(IN);

    return %rsq;
}


sub compute_std
{
    my ($refmod, $file, $len)=@_;
    my %std=();
    my %sum1=();
    for(my $i=1; $i<=$len; $i++)
    {
	$std{$i, 1}=0;
	$std{$i, 2}=0;
	$sum1{$i}=0;
    }

    my $nr=0;
    my %ref=();
    open(REF, "$refmod");
    while(my $line = <REF>)
    {
	if($line =~ /^ATOM/)
	{    
	    next if(substr($line, 12, 4) ne " CA ");
	    $nr++;
	    $ref{$nr, 1} = substr($line,30,8);
	    $ref{$nr, 2} = substr($line,38,8);
	    $ref{$nr, 3} = substr($line,46,8);
	    
	    #printf OUT "%10.3f %10.3f %10.3f\n", $ref{$nr, 1}, $ref{$nr, 2}, $ref{$nr, 3};
	}
    }
    close(REF);


    my $nstr1=0;    
    open(CLU, $file);
    while(my $line=<CLU>)
    {
	if($line=~ /^\s*(\d+)\s+\S+/)
	{
	    my $Lch=$1;
	    $nstr1++;
	    for(my $j=1; $j<=$Lch; $j++)
	    {
		$line=<CLU>;
		if($line =~ /(\S+)\s+(\S+)\s+(\S+)/)
		{
		    my $x=$1; 
		    my $y=$2;
		    my $z=$3;		
		    
		    my $d2= &dist2($ref{$j, 1}, $ref{$j, 2}, $ref{$j, 3}, $x, $y, $z);			
		    $std{$j, 1} += $d2;
		    $sum1{$j} += sqrt($d2);
		}
		else
		{
		    print "wrong format x y z in CLU\n";
		    exit;
		}
	    }		    
	}
    }
    close(CLU);

    if($nstr1>0)
    {
	for(my $i=1; $i<=$len; $i++)
	{	    
	    my $a1= $std{$i, 1}/$nstr1;
	    my $a2= ($sum1{$i}/$nstr1) ** 2;
	    $std{$i, 1} = sqrt($a1);
	    $std{$i, 2} = sqrt(abs($a1 - $a2));
	    $std{$i, 3} = sqrt($a2);	
	}
    }

    return %std;
}

sub dist2
{
    my ($x, $y, $z, $x1, $y1, $z1)=@_;

    my $d=($x-$x1) ** 2 + ($y-$y1) ** 2 + ($z-$z1) ** 2;
    
    return $d;
}

sub dist
{
    my ($x, $y, $z, $x1, $y1, $z1)=@_;

    my $d=sqrt(($x-$x1) ** 2 + ($y-$y1) ** 2 + ($z-$z1) ** 2);
    
    return $d;
}

sub read_threading_cov
{
    my ($init, $len, $topN)=@_;

  
    my %tname=();
    my %all=();
    my %covt=();
    open(INIT, $init);
    my $line =<INIT>;
    my $N=0;
    if($line =~ /^\s*(\S+)/)
    {
	$N=$1;
    }
    for(my $n=1; $n<=$N; $n++)
    {
	for(my $k=1; $k<=$len; $k++)
	{
	    $all{$n, $k}="-";
	}
	
	$line =<INIT>;
	if($line =~ /(\S+)\s+(\S+)\s+\d+\s+(\S+)\s+(\S+)/)
	{
	    my $Lali=$1;
	    my $cscore=$2;
	    my $template=$3;
	    my $prog=$4;
	    $tname{$n, 1}=$template;
	    $tname{$n, 2}=$cscore;
	    $tname{$n, 3}=$prog;
	    $tname{$n, 4}=$Lali;
	    
	    for(my $i=1; $i<=$Lali; $i++)
	    {	
		$line =<INIT>;
		
		if($line =~ /^ATOM/)
		{
		    my $res_no    = substr($line, 22, 4); $res_no = sprintf("%d", $res_no);
		    my $res_name  = "UNK";
                    if($prog ne "WWW")
                    {
                        $res_name  = substr($line, 60, 3);
                    }
                    else
                    {
                        if(length $line >=63)
                        {
                            $res_name=substr($line, 60, 3);
                            if(!exists $ts{$res_name})
                            {
                                $res_name="UNK";
                            }
                        }
                    }

		    #print "$res_name\n";
		    $all{$n, $res_no}=$ts{$res_name};
		}		
	    }
	    <INIT>; #TER
	}
	if($n==$topN)
	{
	    %covt=&compute_cov($len, $n, \%all);
	}	
    }
    close(INIT);

    my %cova=&compute_cov($len, $N, \%all);
    return (\%covt, \%cova);
}

sub compute_cov
{
    my ($len, $N, $all_ref)=@_;
    my %all=%$all_ref;
    my %cov=();
    for(my $k=1; $k<=$len; $k++)
    {
	$cov{$k}=0;
    }

    return %cov if($N<=0);
    
    
    for(my $n=1; $n<=$N; $n++)
    {
	for(my $k=1; $k<=$len; $k++)
	{
	    $cov{$k}++ if($all{$n, $k} ne "-");
	}
    }
    for(my $k=1; $k<=$len; $k++)
    {
	$cov{$k} /=$N;
	#$cov{$k} = 0.05 if($cov{$k}<0.05);  #To make the figure show SS
    }

    return %cov;
}


sub read_threading_dist
{
    my ($init, $len, $top, $model, $target)=@_;
    
    my %tname=();
    my %all=();
    open(INIT, $init);
    my $line =<INIT>;
    my $N=0; 
    if($line =~ /^\s*(\S+)/)
    {
        $N=$1;
    }
    if($top>0)
    {
	$N=$top if($N>$top);
    }

    my $workdir="/tmp/$user/init_$target";
    `mkdir -p $workdir`;
    `rm -fr $workdir/*`;
    `cp $model $workdir/model1.pdb`;

    for(my $n=1; $n<=$N; $n++)
    {
        for(my $k=1; $k<=$len; $k++)
        {
            $all{$n, $k}="-";
        }

        $line =<INIT>;
        if($line =~ /(\S+)\s+(\S+)\s+\d+\s+(\S+)\s+(\S+)/)
        {
            my $Lali=$1;
            my $cscore=$2;
            my $template=$3;
            my $prog=$4;
            $tname{$n, 1}=$template;
            $tname{$n, 2}=$cscore;
            $tname{$n, 3}=$prog;
            $tname{$n, 4}=$Lali;

	    open(TMP, ">$workdir/t$n.pdb");
            for(my $i=1; $i<=$Lali; $i++)
            {   
                $line =<INIT>;

                if($line =~ /^ATOM/)
                {
		    print TMP "$line";
                    my $res_no    = substr($line, 22, 4); $res_no = sprintf("%d", $res_no);
                    my $res_name  = "UNK";
                    if($prog ne "WWW")
                    {
                        $res_name  = substr($line, 60, 3);
                    }
                    else
                    {
                        if(length $line >=63)
                        {
                            $res_name=substr($line, 60, 3);
                            if(!exists $ts{$res_name})
                            {
                                $res_name="UNK";
                            }
                        }
                    }
                    $all{$n, $res_no}=$ts{$res_name};
                }
            }
            <INIT>; #TER
	    close(TMP);
        }
    }
    close(INIT);

    my %std1=();
    #print "$top\n";
    if($top>0)
    {	
	my %count=();
	my $maxd=10;
	for(my $k=1; $k<=$len; $k++)
	{
	    $count{$k}=0;
	    $std1{$k}=0;
	}
	$N=10 if($N>10);

	##model to templates
	for(my $k=1; $k<=$len; $k++)
	{
	    $count{$k}=0;
	}

	for(my $n=1; $n<=$N; $n++)
	{		   	    
	    my %dist=&superpose("$workdir/t$n.pdb", "$workdir/model1.pdb", $len);
		
	    for(my $k=1; $k<=$len; $k++)
	    {
		if(exists $dist{$k})
		{
		    $count{$k}++;
		    $std1{$k} +=$dist{$k};
		}
	    }
	}
		
	for(my $k=1; $k<=$len; $k++)
	{
	    if($count{$k}>0)
	    {
		$std1{$k} /= $count{$k};
	    }
	    else
	    {
		$std1{$k}=$maxd;
	    }
	}
    }    
  
    `rm -fr $workdir`;
    return (\%std1);
}


sub superpose
{
    my  ($pdb1, $pdb2, $len)=@_;
    my @tra=();
    my @rot=();
    my @rst=`$bindir/TMscore $pdb1 $pdb2 -l $len`;
    
    my %dist=();

    if(@rst<26)
    {
	#print "wrong TMscore running: TMscore $pdb1 $pdb2\n";
	return %dist;
    }

    if($rst[17] =~ /TM-score    =\s+(\S+)/)
    {
	my $tm=$1;
	if($tm<0.17)
	{
	    #print "TMscore is too small($tm)\nTMscore $pdb1 $pdb2\n"; #exit;
	    return %dist;	    
	}
    }


    if($rst[24] =~/\s+1\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
    {
	$tra[0]=$1;
	$rot[0][0]=$2;
	$rot[0][1]=$3;
	$rot[0][2]=$4;
	#printf "1 %.5f,%.5f,%.5f,%.5f\n", $1,$2,$3,$4;
    }

    if($rst[25] =~/\s+2\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
    {
	$tra[1]=$1;
	$rot[1][0]=$2;
	$rot[1][1]=$3;
	$rot[1][2]=$4;
	#printf "2 %.5f,%.5f,%.5f,%.5f\n", $1,$2,$3,$4;   
    }
    if($rst[26] =~/\s+3\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
    {
	$tra[2]=$1;
	$rot[2][0]=$2;
	$rot[2][1]=$3;
	$rot[2][2]=$4;
	#printf "3 %.5f,%.5f,%.5f,%.5f\n", $1,$2,$3,$4;
    }
   

    my @pdb=`cat $pdb1`;
    my %xyz1=();

    foreach my $r(@pdb)
    {
        chomp($r);
        if($r =~ /^ATOM/)
        {
	    next if(substr($r, 12, 4) ne " CA ");
            my $x = substr($r, 30, 8);
            my $y = substr($r, 38, 8);
            my $z = substr($r, 46, 8);

            my $rx = $rot[0][0]*$x + $rot[0][1]*$y + $rot[0][2]*$z + $tra[0];
            my $ry = $rot[1][0]*$x + $rot[1][1]*$y + $rot[1][2]*$z + $tra[1];
            my $rz = $rot[2][0]*$x + $rot[2][1]*$y + $rot[2][2]*$z + $tra[2];

	    my $resno=substr($r, 22, 4);
	    $resno =~ s/\s+//g;	    
	    $xyz1{$resno, 1}=$rx;
	    $xyz1{$resno, 2}=$ry;
	    $xyz1{$resno, 3}=$rz;
        }
    }


    @pdb=`cat $pdb2`;
    my %xyz2=();
    foreach my $r(@pdb)
    {
        chomp($r);
        if($r =~ /^ATOM/)
        {
	    next if(substr($r, 12, 4) ne " CA ");
            my $x = substr($r, 30, 8);
            my $y = substr($r, 38, 8);
            my $z = substr($r, 46, 8);

	    my $resno=substr($r, 22, 4);
	    $resno =~ s/\s+//g;	    
	    $xyz2{$resno, 1}=$x;
	    $xyz2{$resno, 2}=$y;
	    $xyz2{$resno, 3}=$z;
	}
    }
        
    for(my $k=1; $k<=$len; $k++)
    {
	if(exists $xyz1{$k, 1} && exists $xyz2{$k, 1})
	{
	    $dist{$k}=&eucl($xyz1{$k, 1}, $xyz1{$k, 2}, $xyz1{$k, 3}, $xyz2{$k, 1}, $xyz2{$k, 2}, $xyz2{$k, 3});
	}
	else
	{
	    #print "$k\n";exit;
	}
    }    

    return %dist;
}
sub eucl
{
    my ($x1, $y1, $z1, $x2, $y2, $z2)=@_;
    my $dist =sqrt(($x1 - $x2)**2 + ($y1 - $y2)**2 + ($z1 - $z2)**2);
    return($dist);
}



sub make_ss
{
    my ($model, $Lch)=@_;

    my %HEC=('H'=>'H', 'G'=>'H', 'I'=>'H', 'B'=>'E', 'b'=>'E', 'E'=>'E', 'T'=>'C', 'C'=>'C');
    my %ss=();


    my @buffer=`cat $model`;
    open(TMP, ">model_tmp.pdb");
    foreach my $r(@buffer)
    {
	print TMP "$r" if($r =~ /^ATOM/);
    }
    close(TMP);


    my @rst=`./stride model_tmp.pdb`;
    my $len=0;
    foreach my $r(@rst)
    {
        if($r =~ /^ASG/)
        {
            my $resno=sprintf "%d", substr($r, 11, 4); #original residue number
            my $s=$HEC{substr($r, 24, 1)};
	    my $a=substr($r, 64, 5);
	    my $res=$ts{substr($r, 5, 3)};
	    $ss{$resno, 1}=$s;
	    my $a1=sprintf "%.2f", $a/$SA{$res};;
	    $a1=1 if($a1>1);   
	    $ss{$resno, 2}=$a1;

	    $len++;
        }
    }

    if($len != $Lch)
    {
	print "Warning: length in $model ($len) is not the same as in $ssf ($Lch), reassign\n";
	
    }
    return %ss;
}


sub BFP
{
    my ($Lch, $init, $profile, $ssfile, $expfile)=@_;
    my $window=4;
    my %pro=();
    my %bfp=();
    my @rst=`cat $ssfile`;
    my $len=0;




    ##predict relative solvent accessibility by spineX
    `cp $profile seq.pssm`; #must be with name *.pssm
    `/nfs/amino-library/SPARKX/sparks-x/SPINE-X/spineX.pl seq.pssm`; #generate seq.phipsi
    my $expfile1="seq.phipsi";
    if(!-s $expfile1)
    {
	die "spineX fail\n";
    }   

    my %HEC=("C"=>1, "H"=>2, "E"=>4);

    my $coil=0;

    foreach my $line(@rst)
    {
	if($line =~ /\s*(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/)
	{	    
	    $bfp{$1, 1}=$HEC{$3};

	    $pro{$1, 1}=$4;	    
	    $pro{$1, 2}=$5;
	    $pro{$1, 3}=$6;
	    $len++;
	    if($len != $1)
	    {
		print "wrong format of $ssf\n";
		exit;
	    }

	    if($bfp{$1, 1} == 1)
	    {
		$coil++;
	    }
	}
    }

    if($len != $Lch)
    {
	print "wrong length\n";
	return;
    }

    my %sas=();    
    open(EXP,"<$expfile");
    <EXP>;
    while(my $line2=<EXP>)
    {
	my @exps =split(/\s+/,$line2);
	my $seqno=$exps[1];	
	my $exp=0;
	for(my $i=2;$i<$#exps;$i++)
	{
	    $exp += ($exps[$i]*0.05);
	}
	$sas{$seqno}=sprintf("%.2f",$exp);
	$bfp{$seqno, 2}=$exps[6];
    }
    close(EXP);


    my %tbf=&TBF($init, $len);


    for(my $i=1; $i<=$len; $i++)
    {
	$pro{$i, 4}=$tbf{$i, 1};
	$pro{$i, 5}=$tbf{$i, 2}; 	
	$pro{$i, 6}=$sas{$i};
    }


    my %rsa=();
    my $i=0;
    open(PRO, $expfile1);
    <PRO>;
    while(my $line=<PRO>)
    {
	chomp($line);  
	if($line =~ /\S+/)
	{
	    $line =~ s/^\s+//g;
	    my @spine=split(/\s+/, $line);	    
	    $i++;
	    $pro{$i, 7}=$spine[10]/$SA{$spine[1]};
	    $rsa{$i}=$pro{$i, 7};
	}
    }  
    close(PRO);
    if($i!=$len)
    {
	print "warning: $i!=$len in RSA by spine-X\n";	
    }

    my $e=7;
    
=pod
    open(PRO, $profile);
    <PRO>;<PRO>;<PRO>;
    while(my $line=<PRO>)
    { 
	chomp($line);  
	if($line =~ /^\s*(\d+)\s+(\S+)\s+(.+)/)
	{
	    my $id=$1;
	    my $r=$3;
	    
	    $i++;
	    if($i!=$id)
	    {
		print "wrong pssm\n";
	    }
	    my @a=split(/\s+/, $r);
	    for(my $j=0; $j<20; $j++)
	    {
		my $k=$j+$e;		
		$pro{$i, $k}=1/(1+exp(-$a[$j]));
	    }	    
	}
    }  
    close(PRO);
    if($i!=$len)
    {
	print "warning: $i!=$len\n";
	return;
    }
=cut
    
    my @features=();
    my $start=1;  
    my $dim=$e;
    for(my $j=1; $j<=$dim; $j++) 
    {
	$pro{0, $j}=0;
    }    

    open(FFF, ">features_bf.dat");
    ##collect features now
    for(my $i=1; $i<=$len; $i++)
    {
	my $i1=$i;
	for(my $j=$start; $j<=$dim; $j++)
	{
	    push(@features, $pro{$i1, $j});
	}
	
	for(my $w=1; $w<=$window; $w++)
	{
	    my $i1=$i-$w;
	    if($i1<1 || $i1 >$len)
	    {
		$i1=0;
	    }
	    
	    for(my $j=$start; $j<=$dim; $j++)
	    {
		push(@features, $pro{$i1, $j});
	    }

	    $i1=$i+$w;
	    if($i1<1 || $i1 >$len)
	    {
		$i1=0;
	    }	    
	    for(my $j=$start; $j<=$dim; $j++)
	    {
		push(@features, $pro{$i1, $j});
	    }
	}
	printf FFF "0 ";
	my $k=1;
	foreach my $f(@features)
	{	    
	    printf FFF "%d:%.2f ", $k, $f;	    
	    $k++;
	}	
	print FFF "\n";
	@features=();
    }
    close(FFF);


    my $mu=20 + 0.5*20*($coil/$Lch);
    my $sigma=$mu/3;


    my $modelname="svm_model_bfp_noTM";
    print "predict B-factor with svm...\n";

    `./svm_classify features_bf.dat $modelname bf.pred >ttt`;
	
    
    my @pred=`cat bf.pred`;
    chomp(@pred);
    open(STD, ">BFP.dat");
    for(my $i=1; $i<=$Lch; $i++)
    {
	my $j=$i-1;
	$bfp{$i, 3}=$pred[$j];
	my $rbf= $pred[$j]*$sigma+$mu;
	$bfp{$i, 4}=$rbf;
	
	printf STD "%d\t%.3f\t%.3f\n",  $i, $pred[$j], $rbf;
    }
    close(STD);
    
    #`cp BFP.dat $cluster_dir/`;

    return (\%rsa, \%bfp);
}

#template-based BFP with init.dat
sub TBF
{
    my ($init, $Lch)=@_;
  
    my $count=0;
    
    open(IN0, $init) or die "$init is missed\n";
    my $line=<IN0>;
    my $first=$line;
    chomp($first);
    if($line =~ /\s*(\S+)\s+(\S+)\s+(\d+)\s+(\d+)/)
    {
	my $n_temp = $1;
	my $type   = $2;
	my $n_good = $3;
	my $n_ori  = $4;
    }
    else
    {
	print "wrong $init format at $line\n";
	return;
    }
    my %all=();

    while($line = <IN0>)
    {    
	chomp($line);
	if($line =~ /\s*(\d+)\s+(\S+)\s+(\d+)\s+(\S+)\s+(\S+)/)
	{	    
	    my $Lali = $1;
	    my $zs   = $2;
	    my $top  = $3;
	    my $pdb  = $4;
	    my $prog = $5;

	    my %align=();
	    my $tseq="";

	    #print "$pdb\n";
            for(my $i=1; $i<=$Lali; $i++)
            {
                $line =<IN0>;

                if($line =~ /^ATOM/)
                {
                    my $res_no    = substr($line, 22, 4); $res_no = sprintf("%d", $res_no);
                    my $res_name  = "UNK";
                    #$res_name  = substr($line, 60, 3) if($prog ne "WWW");
                    $res_name  = substr($line, 60, 3) if($prog ne "WWW");
                    if($prog ne "WWW")
                    {
                        $res_name  = substr($line, 60, 3);
                    }
                    else
                    {
                        if(length $line >=63)
                        {
                            $res_name=substr($line, 60, 3);
                            if(!exists $ts{$res_name})
                            {
                                $res_name="UNK";
                            }
                        }
                    }
                    #print "$res_name\n";
                    $align{$res_no}=$i;
		    $tseq .= $ts{$res_name} if($res_name ne "UNK");
                }
            }
            <IN0>; #TER

	    next if(length $tseq <5);
	    my $tseq1=$tseq;
	    $tseq1 =~ s/X//g;	    
	    next if($tseq1 eq "");

	    my $tname=$pdb;
	    $tname =~ s/_//g;
	    next if(length $tname < 5);
	    my $id=lc(substr($pdb, 0, 4)) . substr($pdb, 4, 1);
	    next if(!-s "$libdir/$id.bf");

	    my @rst=`cat $libdir/$id.bf`;
	    next if(@rst<2);
	    chomp(@rst);

	    my ($flag, $pbf_ref) = &assign_BF(\%align, $tseq, $rst[0], $rst[1], $Lch);    
	    my %pbf=%$pbf_ref;
	    next if($flag==0);
	    $count++;   
	    for(my $i=1; $i<=$len; $i++)
	    {
		$all{$count, $i}=$pbf{$i};
	    }	 
	}
	else
	{
	    print "wrong $init format at $line\n";
	    return;
	}  
	
    }
    close(IN0); 

    my %tbf=();
    for(my $i=1; $i<=$len; $i++)
    { 
	my $j=0;
	my $sum=0;
	for(my $n=1; $n<=$count; $n++)
	{	
	    if($all{$n, $i} != 0)
	    {
		$j++;
		$sum += $all{$n, $i};
	    }	    
	}    
	if($j!=0)
	{
	    $sum /=$j;
	}
	my $cov=0;
	if($count>0)
	{
	    $cov=$j/$count;
	}
	$tbf{$i, 1}=$cov;
	$tbf{$i, 2}=$sum;	
    }

    return %tbf;
}

sub assign_BF
{
    my ($align_ref, $tseq, $seq0, $bf0, $len)=@_;

    my $flag=0;
    my %align=%$align_ref;
    my @bfactor0=split(" ", $bf0);

    my %pbf=();

    my @bfactor=&normalize_by_zscore(\@bfactor0);


    if(@bfactor<@bfactor0)
    {
	return ($flag, \%pbf);
    }


    my @rst=`./align $tseq $seq0 3`;   
    #print `./align $tseq $seq0 3`;#exit;

    my $alignQ=$rst[7];
    chomp $alignQ;
    my $alignM=$rst[8];
    chomp $alignM;
    my $alignT=$rst[9];
    chomp $alignT;

    my @seqQ=();   ### alignment of query seq
    my @seqT=();   #### alignment of target seq
    my @seqM=();   ### alignment of matches;

    for(my $i=0; $i<length $alignQ; $i++)
    {
	my $ch=substr($alignQ, $i, 1);
	push(@seqQ, $ch);

	$ch=substr($alignT, $i, 1);
	push(@seqT, $ch);

	$ch=substr($alignM, $i, 1);
	push(@seqM, $ch);
    }


    my %tbf=();
    my $q=0;
    my $t=0;
    for(my $i=0; $i<@seqQ; $i++)
    {
	#print "$i ";
	my $j=$i+1;

	if($seqQ[$i] ne "-")
	{
	    $q++; #order in aligned template
	}
	if($seqT[$i] ne "-")
	{
	    $t++; #order in original template
	}
	
	if($i>=@seqM)
	{
	    exit;
	}
	if($seqM[$i] eq ":")
	{	    
	    $tbf{$q}=$bfactor[$t-1];
	}
    }   
    #print "\n";
        
    for(my $i=1; $i<=$len; $i++)
    {
	if(exists $align{$i})
	{
	    my $q=$align{$i};
	    if(!exists $tbf{$q})
	    {		
		$pbf{$i}=0;
	    }
	    else
	    {
		$pbf{$i}=$tbf{$q};	
		$flag=1;
	    }
	}
	else
	{
	    $pbf{$i}=0;
	}
	#printf "%4d %6.2f\n", $i, $pbf{$i};
    }

    return ($flag, \%pbf);
}


sub normalize_by_max
{
    my ($raw_ref)=@_;
    my @raw=@$raw_ref;

    my @norm=();

    my $max=200;

    foreach my $r(@raw)
    {
	my $n=$r/$max;
	$n=1 if($n>1);
	push(@norm, $n);
    }

    return @norm;    
}


sub normalize_by_zscore
{
     my($score_ref)=@_;
     my @r_score  =@$score_ref;

     my $Lch = @r_score;
     my $z1_a  = 0;
     my $z1_a2=0;

     my @zscore=();
     my $a=$r_score[0];    
     my $flag=0;
     for(my $i=1; $i<@r_score; $i++)
     {
	 if($a!=$r_score[$i])
	 {
	     $flag=1;
	     last;
	 }
     }

     return @zscore if($flag==0);
     
     
     foreach my $i(@r_score)
     {
         $z1_a  += $i;
         $z1_a2 += $i**2;
     }
     $z1_a/=$Lch;
     $z1_a2/=$Lch;
     my $z1_sd = sqrt($z1_a2 - $z1_a**2);



     foreach my $i(@r_score)
     {

         my $s=($i-$z1_a)/$z1_sd;
         $s = sprintf("%.2f", $s);
         push(@zscore, $s);
     }

     return @zscore;
}

